function [p logP gradlogP gradlog1_P] = choiceModel_exp_squared(d, W, K1, K2)
    %
    % [p logP gradlogP gradlog1_P] = choiceModel_exp(d, W, K1, K2)
    %
    % Compute P(confusion | d) with the choice model p = exp(-d^2)/2 and also
    % provide gradient in term of W for log(p) and log(1-p)
    %
    % ********************************
    % Input Parameters:
    % ********************************
    % d : pairwise Euclidean distance. It could be a value, vector, or
    % matrix, p and logP will be in the shape of d. When call this
    % function, if d is available, it is not necessary to provide other
    % parameters.
    % The following three parameters are used for learning weight.
    % W : weight vector
    % K1: the corresponding column in gram matrix for the first object
    % K2: the corresponding column in gram matrix for the second object
    %
    % ********************************
    % Input Parameters:
    % ********************************
    % p : confusion probability
    % logP : log(confusion probability). (Because the exponential shape of  choice model, it sometimes produces numerical problems. 
    % So logP cancels the exponential to avoid such issues.
    % The following two output are used in learning process
    % gradlogP: the gradient of log(p) at W
    % gradlog1_P: the gradient of log(1 - p) at W
    
    if (nargin == 1)
        p = exp(-d.^2)/2;
        logP = -d.^2 - log(2);
    else
        delta = K1 - K2;
        y_difference = W * delta;
        d_squared = y_difference' * y_difference;
        
        p = exp(-d_squared)/2;
        logP = - d_squared - log(2);
        gradlogP = -2 * W * (delta * delta');
        gradlog1_P = gradlogP / (1 - 2 * exp(d_squared));
    end