function [x y a b d W_opt] = generateData(type, T, choiceModel)
    % Generate synthetic data set for experiment
    %
    % Input parameters:
    % type: corresponding data index in the notes
    % T: the value of a + b. If T < 0, then a = p(incorrect|d), b = 1 - a
    % choideModel: a function handle for the choice model confusion probability = \psi(d)
    %
    % Output:
    % x: the coordinates of n stimuli in raw feature space
    % y: the coordinates of n stimuli in perceptual space
    % a/b: the number of the incorrect/correct answers in XAB experiments
    % d: true pairwise distance in perceptual space(type, T, m, seed, choiceModel)
    % W_opt: ground truth w, if available.
    
    seed = 1;
    
    switch(type)
        case '1D_1',
            x = -1:0.2:1;
            y = 2 * exp(-(x + 0.6).^2)  - 3 * exp(-(x + 0.2).^2) + 2 * exp(-(x - 0.8).^2);
            W_opt = zeros(1,11);   W_opt(3) = 4;  W_opt(5) = 7; W_opt(8) = -5;  W_opt(10) = 4;
        case '1D_2',
            x = -1:0.2:1;
            y = cos(5*pi.*x) * 0.6  + x * 0.5;
            W_opt = zeros(1,11);
        case '1D_3',
            x = -1:0.2:1;
            y = exp(2 * (x + 1))/3;
            W_opt = zeros(1,11);
        case '2D_1',
            [x1 x2] = meshgrid(-1:0.4:1, -1:0.4:1);
            x = [x1(:) x2(:)]';
            y1 = 6 * exp(-(x(1,:) + 1).^2 - (x(2,:) + 1).^2);
            y2 = 6 * exp(-(x(1,:) - 1).^2 - (x(2,:) - 1).^2);
            y = [y1; y2];
            W_opt = zeros(2,36); W_opt(1,1) = 6; W_opt(2,36) = 6;
        case '2D_2',
            [x1 x2] = meshgrid(-1:0.4:1, -1:0.4:1);
            x = [x1(:) x2(:)]';
            y1 = 2 * tanh(x(1,:) + 2 * x(2,:) + 0.1);
            y2 = 4 * tanh(2.5 * x(1,:) - x(2,:) + 0.8);
            y = [y1; y2];
            W_opt = zeros(2,36);
        case '2D_3',
            [x1 x2] = meshgrid(-1:0.4:1, -1:0.4:1);
            x = [x1(:) x2(:)]';
            y1 = 0.1*(- 0.3*x(1,:) + 0.5 * x(2,:) + 1.5).^4 - 0.08*(0.5*x(1,:) - 0.3 * x(2,:) + 1.5).^4 ;
            y2 = 0.08*(- 0.6*x(1,:) + 0.3 * x(2,:) + 1.5).^4 - 0.1*(0.3*x(1,:) + 0.5 * x(2,:) + 1.5).^4 ;
            y = [y1; y2];
            W_opt = zeros(2,36);
         case '1D2D',
            x = -1:0.2:1;
            y1 = cos(x)*2;
            y2 = sin(x)*2;
            y = [y1; y2];
            W_opt = zeros(1,11);
    end
    
    % compute pair-wise distance
    d = squareform(pdist(y'));
    
    % confusion probability function p(incorrect|d)
    prob  = feval(choiceModel, d);
    
    % draw a ~ Binomial(T,prob). If T < 0, a = prob.
    if T < 0,
        a = tril(prob, -1) ;
        b = tril(1 - prob, -1);
    else
        rand('state', seed);
        a = tril(binornd(T, prob), -1);
        b = tril(T - a, -1);
    end
    
    % For 2D data set, only keep the pairs with pairwise distance is smaller than or equal to 0.8
    if (strcmp(type(1), '2'))
        mask = squareform(pdist(x')) <= 0.8 ;
        a = a .* mask;
        b = b .* mask;
    end
    a = a + a';
    b = b + b';
    
    